<?php

/**
 * SHD SM-VIEW File Version API
 *
 * @package    idoit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_api_model_shd_file_version implements isys_api_model_interface
{
    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'file_version';

    /**
     * Modules DAO.
     *
     * @var isys_module_dao
     */
    protected $dao;

    /**
     * Constructor.
     */
    function __construct()
    {
        $this->dao = isys_cmdb_dao_category_s_file::instance(isys_application::instance()->database);
    }

    /**
     * Mapping of database and API keys of file.
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_cats_file_list__id'                   => 'id',
        'isys_cats_file_list__isys_obj__id'         => 'obj_id',
        'isys_obj__title'                           => 'obj_title',
        'isys_file_version__id'                     => 'version_id',
        'isys_file_version__title'                  => 'version_title',
        'isys_file_version__description'            => 'version_description',
        'isys_file_version__revision'               => 'version_revision',
        'isys_file_version__isys_file_physical__id' => 'physical_file_id',
        'isys_filename'                             => 'physical_file_name',
        'isys_physical_filename'                    => 'physical_file_filename',
        'isys_file_category__id'                    => 'category_id',
        'isys_file_category__title'                 => 'category_title',
        'isys_file_category__description'           => 'category_description',
    );

    /**
     * Mapping of database and API keys of version.
     *
     * @var array
     */
    protected $version_db_to_api_map = array(
        'isys_file_version__id'                     => 'id',
        'isys_file_version__title'                  => 'title',
        'isys_file_version__description'            => 'description',
        'isys_file_version__revision'               => 'revision',
        'isys_file_version__isys_file_physical__id' => 'physical_file_id',
    );

    /**
     * Read file objects.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int|null,
     * )
     *
     * @return array
     */
    public function read($p_params)
    {
        $obj_id = isset($p_params["id"]) ? $p_params['id'] : null;
        $include_file = isset($p_params['include_file']) ? $p_params['include_file'] : null;
        $res = $this->dao->get_data(null, $obj_id);
        return $this->_map_result_to_array($res, $this->db_to_api_map, $include_file);
    }

    /**
     * List version objects.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int|null,
     * )
     *
     * @return array
     */
    public function versions($p_params)
    {
        $obj_id = isset($p_params['id']) ? $p_params['id'] : null;
        $include_file = isset($p_params['include_file']) ? $p_params['include_file'] : null;
        $res = $this->dao->get_versions_by_obj_id($obj_id);
        return $this->_map_result_to_array($res, $this->version_db_to_api_map, $include_file);
    }

    /**
     * Map database results to array.
     *
     * @param  isys_component_dao_result $res
     * @param  array                     $mapping
     * @param  boolean                   $include_file
     * @return array
     */
    protected function _map_result_to_array($res, $mapping, $include_file = false)
    {
        $objects = array();
        while ($row = $res->get_row()) {
            $obj = array();
            foreach ($mapping as $db_key => $api_key) {
                $obj[$api_key] = isset($row[$db_key]) ? $row[$db_key] : null;
            }

            if (! empty($obj['physical_file_id'])) {
                $file = $this->dao->get_filemanager_dao_by_physical_file_id($obj['physical_file_id']);
                $file_row = $file->get_row();
                $obj['physical_file_name'] = $file_row['isys_filename'];
                $obj['physical_file_filename'] = $file_row['isys_physical_filename'];

                $obj['url'] = isys_helper_link::create_url([
                    C__GET__MODULE_ID    => C__MODULE__CMDB,
                    C__GET__FILE_MANAGER => 'get',
                    C__GET__FILE__ID     => $obj['physical_file_id'],
                ]);
                if ($include_file) {
                    $obj['file'] = $this->_get_file($file_row['isys_physical_filename']);
                }
            }
            $objects[] = $obj;
        }
        return $objects;
    }

    /**
     * Get base64 encoded file.
     *
     * @param  string $filename
     * @return string
     */
    private function _get_file($filename)
    {
        $path_upload =$this->_get_upload_path($filename);
        $content = file_get_contents($path_upload);
        return base64_encode($content);
    }

    /**
     * Create new file.
     *
     * @param $p_params
     *
     * $p_param structure:
     * array (
     *     'obj_id'      => int,
     *     'category_id' => int,
     *     'title'       => string,
     *     'description' => string,
     *     'file'        => string,
     *     'filename'    => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        global $g_comp_session;

        $required_params = array(
            'obj_id',
            'title',
            'file',
            'filename',
        );

        // validate params
        try {
            $this->check_required_parameters($required_params, $p_params['data']);
        } catch (isys_exception_validation $ex) {
            return $this->return_error($this->module_name, 'create', 'could not create object - ' . $ex->getMessage());
        }

        $data = $p_params['data'];
        $obj_id = $data['obj_id'];

        // get unique filename
        $file_prefix = $obj_id . '__' . time() . '__';
        $filename = $data['filename'];
        $unique_filename = $file_prefix . $filename;

        // save file
        $upload_path = $this->_save_file($unique_filename, $data['file']);
        $md5_hash = md5_file($upload_path);

        // create database entries
        $l_version_id = null;
        $l_physical_id = $this->dao->create_physical_file($unique_filename, $filename, $md5_hash, $g_comp_session->get_user_id());

        if ($l_physical_id > 0) {
            // add new version
            $description = isset($data['description']) ? $data['description'] : null;
            $l_version_id = $this->dao->create_version($obj_id, $l_physical_id, $data['title'], $description);
        }

        // update connectors
        $l_cats_id = $this->dao->create_connector('isys_cats_file_list', $obj_id);

        $category_id = isset($data['category_id']) ? $data['category_id'] : null;
        $this->dao->update_cats_file_list($l_cats_id, $l_version_id, $category_id, '');
        return $this->return_success($this->module_name, 'create', $l_cats_id);
    }

    /**
     * Save base64 encoded string to file.
     *
     * @param  $filename string
     * @param  $content string
     * @return string
     */
    private function _save_file($filename, $content)
    {
        $path_upload =$this->_get_upload_path($filename);
        $f = fopen($path_upload, 'wb');
        fwrite($f, base64_decode($content));
        fclose($f);

        return $path_upload;
    }

    /**
     * Get upload path, depending on i-doit version.
     *
     * @param  string $filename
     * @return string
     */
    private function _get_upload_path($filename)
    {
        global $g_dirs;

        $app = isys_application::instance();
        if (method_exists($app, 'getOrCreateUploadFileDir')) {
            $path_upload = $app->getOrCreateUploadFileDir($filename) . $filename;
        } else {
            $path_upload = realpath($g_dirs['fileman']['target_dir']) . DS . $filename;
        }
        return $path_upload;
    }

    /**
     * Update meta data of file.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'          => int,
     *     'title'       => string,
     *     'description' => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $required_params = array('id');

        // validate params
        try {
            $this->check_required_parameters($required_params, $p_params);
        } catch (isys_exception_validation $ex) {
            return $this->return_error($this->module_name, 'update', 'could not update object - ' . $ex->getMessage());
        }

        // update version
        $dao = isys_cmdb_dao_category_s_file_version::instance(isys_application::instance()->database);
        $data = $p_params['data'];
        $props = [];
        if (isset($data['title'])) {
            $title = ! empty($data['title']) ? $data['title'] : null;
            $props[] = 'isys_file_version__title = ' . $dao->convert_sql_text($title);
        }
        if (isset($data['description'])) {
            $description = ! empty($data['description']) ? $data['description'] : null;
            $props[] = 'isys_file_version__description = ' . $dao->convert_sql_text($description);
        }

        $prop_string = implode(',', $props);
        $l_sql = "UPDATE isys_file_version SET $prop_string
                WHERE isys_file_version__id = " . $dao->convert_sql_id($p_params['id']) . ";";

        try {
            $dao->update($l_sql) && $dao->apply_update();
        } catch (isys_exception $ex) {
            return $this->return_error($this->module_name, 'update', 'could not update object - ' . $ex->getMessage());
        }
        return $this->return_success($this->module_name, 'update', $p_params['id']);
    }

    /**
     * Delete file version.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        $required_params = array('id');

        // validate params
        try {
            $this->check_required_parameters($required_params, $p_params);
        } catch (isys_exception_validation $ex) {
            return $this->return_error($this->module_name, 'delete', 'could not delete object - ' . $ex->getMessage());
        }

        $dao = isys_cmdb_dao_category_s_file_version::instance(isys_application::instance()->database);
        $file = $dao->get_file_by_version_id($p_params['id'])->get_row();

        $dao->delete_file($file['isys_file_physical__filename']);
        $p_id = $file['isys_file_physical__id'];
        try {
            $dao->update('DELETE FROM isys_file_version WHERE isys_file_version__id = ' . $dao->convert_sql_id($p_params['id']) . ';');
            $dao->update('DELETE FROM isys_file_physical WHERE isys_file_physical__id = ' . $dao->convert_sql_id($p_id) . ';');
        } catch (isys_exception $ex) {
            return $this->return_error($this->module_name, 'delete', 'could not delete object - ' . $ex->getMessage());
        }
        return $this->return_success($this->module_name, 'delete', $p_params['id']);
    }

    /**
     * Return success message.
     *
     * @param string $module
     * @param string $action
     * @param int|null $returned_id
     * @return array
     */
    protected function return_success($module, $action, $returned_id)
    {
        return array(
            'message'      => "$module.$action.success",
            'id'           => $returned_id,
            'current_date' => date('Y-m-d'),
        );
    }

    /**
     * Return error message.
     *
     * @param string $module
     * @param string $action
     * @param string $message
     * @return array
     */
    protected function return_error($module, $action, $message)
    {
        return array(
            'message'      => "$module.$action.error",
            'error'        => $message,
            'current_date' => date('Y-m-d'),
        );
    }

    /**
     * Check if request contains required parameters.
     *
     * @param array $required_parameters
     * @param array $request
     * @throws \isys_exception_validation
     */
    protected function check_required_parameters($required_parameters, $request)
    {
        $missing_parameters = [];
        foreach ($required_parameters as $param) {
            if (! isset($request[$param]) || $request[$param] == '') {
                $missing_parameters[] = $param;
            }
        }

        if (! empty($missing_parameters)) {
            $params = implode(', ', $missing_parameters);
            $message = "following parameters are missing: $params";
            throw new isys_exception_validation($message, $missing_parameters);
        }
    }
}
