<?php

use idoit\Module\License\LicenseServiceFactory;

/**
 * SHD API Licenses.
 *
 * @package SHD
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_licences implements isys_api_model_interface
{

    const LICENSE_FILE = __DIR__ . "/../../../lic.lic";
    const INDEX_FILE = __DIR__ . "/../../../init_index.php";

    private $_license;

    /**
     * License DAO.
     *
     * @var smview_cmdb_dao_lic
     */
    private $_license_dao = null;

    public function __construct()
    {
        global $g_comp_database_system, $g_license_token;

        // get current licensed and used object count
        $licenseService = LicenseServiceFactory::createDefaultLicenseService($g_comp_database_system, $g_license_token);
        $this->_license = [
            "in_use"   => $licenseService->getUsedLicenseObjects(true),
            "objcount" => $licenseService->getTotalObjects(),
        ];
        $this->_license_dao = smview_cmdb_dao_lic::instance(isys_application::instance()->database);
    }

    /**
     * Get current licensed and used object count.
     *
     * @param  array $p_params
     * @return array
     */
    public function read($p_params)
    {
        return $this->_license;
    }

    public function licencesrequest()
    {
        // OLD LICENSE REQUEST:
        //  2 Modul nummer
        //  6 Geheimnis
        //  2 Anzahl
        //  2 Prüfsumme
        // (XXXX-XXXX-XXXX)

        // NEW LICENSE REQUEST:
        //  2 Modul nummer
        //  6 Geheimnis
        //  8 Anzahl
        //  4 Prüfsumme
        // (XXXX-XXXX-XXXX-XXXX-XXXX)

        if (! file_exists(self::INDEX_FILE)) {
            $this->_createIndexFile();
        }
        include self::INDEX_FILE;

        $moduleId = "09";
        $apikey = lic;
        $count = $this->_getTotalObjectCount();

        $licenseRequest = $moduleId . $apikey . $count;
        return $this->_getEncryptedLicenseData($licenseRequest, 5);
    }

    private function _getTotalObjectCount()
    {
        global $g_comp_database_system, $g_license_token;

        // get current licensed object count
        $licenseService = LicenseServiceFactory::createDefaultLicenseService($g_comp_database_system, $g_license_token);
        $totalCount = $licenseService->getTotalObjects();

        // trim count to 8 characters
        $totalCount = strlen($totalCount) > 8 ? substr($totalCount, 0, 8): $totalCount;

        // add leading zeros
        return str_pad($totalCount, 8, '0', STR_PAD_LEFT);
    }

    public function licenceskey($p_params)
    {
        //  4 Ablaufdatum
        //  6 Geheimnis
        //  2 Modul nummer
        //  2 Anzahl

        //  2 Prüfsumme
        // (XXXX-XXXX-XXXX-XXXX )

        if (file_exists(self::INDEX_FILE)) {
            include self::INDEX_FILE;
        }

        // decode license request
        $licenceRequest = "";
        $licenseResponse = $p_params["licence_request"];
        for ($i = 0; $i < strlen($licenseResponse); $i++) {
            if ($licenseResponse[$i] == "-") {
                continue;
            }
            $licenceRequest .= hexdec($licenseResponse[$i]) - 5;
        }

        $moduleId = "07";
        $secret = $licenceRequest[2] . $licenceRequest[3]. $licenceRequest[4] . $licenceRequest[5] . $licenceRequest[6] . $licenceRequest[7];
        $countLength = strlen($licenceRequest) < 16 ? 2 : 8;
        $countId = substr($licenceRequest, 8, $countLength);

        $date = new DateTime('2018-12-31');
        $expireDate =  $date->format('my');

        $licenceKey = $expireDate . $secret . $moduleId . $countId;
        return $this->_getEncryptedLicenseData($licenceKey, 4);
    }

    private function _getEncryptedLicenseData($licenceKey, $offset)
    {
        $return = "";
        $check_sum = 0;
        for ($i = 0; $i < strlen($licenceKey); $i++) {
            if ($i % 4 == 0 && $i != 0) {
                $return .= "-";
            }
            $check_sum += $licenceKey[$i];
            $return .= dechex($licenceKey[$i] + $offset);
        }
        $s_check_sum = (string) $check_sum;

        if (strlen($licenceKey) > 12) {
            // add leading zeros to checksum
            $return .= '-';
            $s_check_sum = str_pad($s_check_sum, 4, '0', STR_PAD_LEFT);
        }

        for ($i = 0; $i < strlen($s_check_sum); $i++) {
            $return .= dechex($s_check_sum[$i] + $offset);
        }
        return $return;
    }

    public function getLicencesKey()
    {
        if (file_exists(self::LICENSE_FILE)) {
            return file_get_contents(self::LICENSE_FILE);
        }
        return false;
    }

    public function activation($p_params)
    {
        // check license key
        if ($this->_license_dao->checkLicenseKey($p_params["licence_key"]) !== true) {
            return "not a LicenseKey";
        }

        // license key is right
        if (file_exists(self::LICENSE_FILE)) {
            unlink(self::LICENSE_FILE);
        }

        $myfile = fopen(self::LICENSE_FILE, "w");
        $txt = $p_params["licence_key"];
        fwrite($myfile, $txt);
        return true;
    }

    public function check_lic()
    {
        include self::INDEX_FILE;

        // check test time
        $datetime1 =  DateTime::createFromFormat('U', time);
        $datetime2 = new DateTime();
        $interval = $datetime1->diff($datetime2);

        if ($interval->d > 31) {
            return ["massage" => "testtime expired"];
        }

        if ($this->_license["in_use"] > $this->_license["objcount"]) {
            return ["massage" => "wrong count"];
        }
        return $this->_license_dao->checkLicenseByModule("Insight");
    }

    public function check_key($p_params)
    {
        return $this->_license_dao->checkLicenseKey($p_params[0]);
    }

    private function _createIndexFile()
    {
        $file = fopen(self::INDEX_FILE, "w");
        $txt = '<?php
                 /*
                  * SHD SM-VIEW license file - do not touch
                  */

                 define("time", "'. time() . '");
                 define("lic",  "'. rand(100000, 999999) . '");
                 ';
        fwrite($file, $txt);
    }

    /**
     * Get license status.
     *
     * This method will be called by SM-VIEW on start-up.
     *
     * @return array
     */
    public function getLicenceStatus()
    {
        return $this->_license_dao->getLicStatus();
    }
}
