<?php

/**
 * shd.api.map_object
 *
 * @package SHD
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_map_object implements isys_api_model_interface
{

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    protected $m_cache_events = null;

    /**
     * Map DAO.
     *
     * @var smview_cmdb_dao_map|null
     */
    private $_map_dao = null;

    public function __construct()
    {
        try {
            $this->m_cache_events = new \isys_cache_smview_memcached();
            // this namespace is also used by:
            //      isys_api_model_shd_map.class.php,
            //      isys_api_model_shd_map_connection.class.php,
            //      isys_api_model_shd_maps.class.php
            //      isys_api_model_shd_map_lineproperty.class.php
            $this->m_cache_events->ns('map_events');
        } catch (isys_exception $ex) {
        }
        $this->_map_dao = smview_cmdb_dao_map::instance(isys_application::instance()->database);
    }

    /**
     * Read map object.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function read($p_params)
    {
        if (isset($p_params["mapobject_id"]) == false) {
            throw new isys_exception_api('Parameter mapobject_id missed', -32602);
        }
        return $this->_map_dao->getMapObject($p_params["mapobject_id"]);
    }

    /**
     * Read map object by map ID.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function readByMap($p_params)
    {
        if (isset($p_params["map_id"]) == false) {
            throw new isys_exception_api('Parameter map_id missed', -32602);
        }
        return $this->_map_dao->getMapObjectByMap($p_params["map_id"]);
    }

    /**
     * Create map object.
     *
     * @param  array $p_params
     * @return array
     * @throws Exception
     */
    public function create($p_params)
    {
        $l_return_id = $this->_map_dao->createMapObject($p_params);
        $this->setEvent($p_params["shd_map__id"], $l_return_id, "create");
        $this->updateLayerIndex(["map_id" => $p_params["shd_map__id"], "mapobject_id" => $l_return_id]);
        return $this->_getSuccessMessage(__FUNCTION__, $l_return_id);
    }

    /**
     * Update map object.
     *
     * @param  array $p_params
     * @return array
     * @throws Exception
     */
    public function update($p_params)
    {
        $this->_map_dao->updateMapObject($p_params);
        $this->setEvent($p_params["shd_map__id"], $p_params["mapobject_id"], "update");
        $this->updateLayerIndex(["map_id" => $p_params["shd_map__id"], "mapobject_id" => $p_params["mapobject_id"]]);
        return $this->_getSuccessMessage(__FUNCTION__);
    }

    /**
     * Delete map object.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function delete($p_params)
    {
        $mapObject = $this->read($p_params);
        $this->_map_dao->deleteMapObject($p_params["mapobject_id"]);

        $this->setEvent($mapObject["shd_map__id"], $p_params["mapobject_id"], "delete");
        return $this->_getSuccessMessage(__FUNCTION__);
    }

    /**
     * Get status of map object.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_cache
     */
    public function status($p_params)
    {
        if (isset($p_params["mapobject_id"]) == false) {
            throw new isys_exception_api('Parameter mapobject_id missed', -32602);
        }

        $statusDao = smview_cmdb_dao_status::instance(isys_application::instance()->database);
        $mapObject = $this->_map_dao->getMapObject($p_params["mapobject_id"]);
        $return = [];
        if ($mapObject["isys_obj__id"] != "0") {
            // get live status
            $m_status = $statusDao->getStatus($mapObject["isys_obj__id"], true);
            if ($m_status !== []) {
                if ($mapObject["livestatus_services"] !== "a:0:{}" && $mapObject["livestatus_services"] !== "") {
                    $searched_services = $mapObject["livestatus_services"];
                    $m_status = $statusDao->get_services($mapObject["isys_obj__id"], $searched_services);
                }
                $return = $m_status;
            } else {
                // get service status
                $bsmServiceStatus = $statusDao->getBsmServiceStatus($mapObject["isys_obj__id"]);
                if ($bsmServiceStatus && $bsmServiceStatus["status"] != "-1") {
                    $return = $bsmServiceStatus;
                }
            }
        }
        return $return;
    }

    private function setEvent($map_id, $mapobject_id, $eventart)
    {
        if (! $this->m_cache_events) {
            return;
        }

        $eventArray = $this->m_cache_events->get("event");
        $date = new DateTime();

        //clear Objects after 60 sek
        foreach ($eventArray as $key => $f_event) {
            //the Element is updated so it's not throwed away after 60 sec
            if ($f_event["timestamp"] + 60 < $date->getTimestamp()) {
                unset($eventArray[$key]);
            }
        }

        foreach ($eventArray as $key => $f_event) {
            if ($f_event["mapobject_id"] == $mapobject_id) {
                $f_event["timestamp"] = $date->getTimestamp();
                $eventArray[$key] = $f_event;
                $this->m_cache_events->set("event", $eventArray, 60);
                return;
            }
        }

        $eventArray[] = ["map_id" => $map_id, "mapobject_id" => $mapobject_id, "timestamp" => $date->getTimestamp(), "eventart" => $eventart];
        $this->m_cache_events->set("event", $eventArray, 60);
    }

    /**
     * Update layer index.
     *
     * @param  array $p_params
     * @return array
     * @throws Exception
     */
    public function updateLayerIndex($p_params)
    {
        //get all Objects
        $l_map = $this->_map_dao->getMap($p_params["map_id"]);
        $l_layerIndex = explode(";", $l_map["layerindex"]);
        if ($l_layerIndex == null) {
            $l_layerIndex = [];
        }

        //check in index and Remove ist
        if (isset($p_params["index"])) {
            $searchIndex = array_search($p_params["mapobject_id"], $l_layerIndex);
            if ($searchIndex !== false) {
                unset($l_layerIndex[$searchIndex]);
            }
            array_splice($l_layerIndex, $p_params["index"], 0, $p_params["mapobject_id"]);
        }
        $l_mapObjects = $this->_map_dao->getMapObjectByMap($p_params["map_id"]);

        // remove when object is removed
        foreach ($l_layerIndex as $layerKey => $f_mapObjectId) {
            $unset = true;
            foreach ($l_mapObjects as $f_mapObject) {
                if ($f_mapObjectId == $f_mapObject["mapobject_id"]) {
                    $unset = false;
                }
            }

            if ($unset) {
                unset($l_layerIndex[$layerKey]);
                $l_layerIndex = array_values($l_layerIndex);
            }
        }

        // add new objects
        foreach ($l_mapObjects as $l_mapObject) {
            $searchIndex = array_search($l_mapObject["mapobject_id"], $l_layerIndex);
            if ($searchIndex === false) {
                $l_layerIndex[] = $l_mapObject["mapobject_id"];
            }
        }

        $l_update["map_id"] = $p_params["map_id"];
        $l_update["layerindex"] = json_encode($l_layerIndex);

        $this->_map_dao->updateMap($l_update);
        return $this->_getSuccessMessage(__FUNCTION__);
    }

    /**
     * Get a common success message.
     *
     * @param  string $methodName
     * @param  int|true $id
     * @return array
     */
    private function _getSuccessMessage($methodName, $id = true)
    {
        return array(
            'message'      => "map_object.$methodName.success",
            'response'     => $id,
            'current_date' => date('Y-m-d')
        );
    }
}
