import * as React from 'react'
import { loadAjax } from './ajax'

export interface UpdaterComposeProps {
  ajax_url: string,
  session: string,
  language: {[key: string]: string},
  spinner_url: string,
  is_update_mode: boolean,
  modules: Module[],
}

interface Module {
  name: string,
  title: string,
  is_updatable?: boolean,
  is_loading?: boolean,
  installed_version: string,
  available_version?: string,
}

interface UpdaterComposeState {
  modules: Module[],
}

export class UpdaterCompose extends React.Component<UpdaterComposeProps, UpdaterComposeState> {
  constructor (props: UpdaterComposeProps) {
    super(props)
    this.state = {
      modules: this.props.modules
    }
    this.getAvailableModuleVersion = this.getAvailableModuleVersion.bind(this)
    this.updateModule = this.updateModule.bind(this)
    this._setModuleProperty = this._setModuleProperty.bind(this)
  }

  componentDidMount () {
    // get all available versions after one second
    setTimeout(() => {
      this.state.modules.forEach(module => {
        this.getAvailableModuleVersion(module.name)
      })
    }, 1000);
  }

  getAvailableModuleVersion (moduleName: string) {
    this._setModuleProperty(moduleName, 'is_loading', true)
    const url = `${this.props.ajax_url}&func=get-available-version&module=${moduleName}`
    loadAjax(url, this.props.session)
      .then(res => {
        const response = { ...res.data }
        this._setModuleProperty(moduleName, 'available_version', response.available_version)
        this._setModuleProperty(moduleName, 'is_updatable', response.is_updatable)
        this._setModuleProperty(moduleName, 'is_loading', false)
      })
      .catch(() => {
        this._setModuleProperty(moduleName, 'is_loading', false)
      })
  }

  updateModule (moduleName: string) {
    this._setModuleProperty(moduleName, 'is_loading', true)
    const url = `${this.props.ajax_url}&func=update&module=${moduleName}`
    loadAjax(url, this.props.session)
      .then(res => {
        const response = { ...res.data }
        this._setModuleProperty(moduleName, 'installed_version', response.installed_version)
        this._setModuleProperty(moduleName, 'is_updatable', false)
        this._setModuleProperty(moduleName, 'is_loading', false)
      })
      .catch(() => {
        this._setModuleProperty(moduleName, 'is_loading', false)
      })
  }

  _setModuleProperty (moduleName: string, property: string, value: string|boolean) {
    const searchModules = this.state.modules.filter(m => m.name == moduleName)
    if (searchModules.length == 0) {
      return
    }

    // update property
    const module = searchModules[0]
    module[property] = value

    const allModules = this.state.modules.filter(m => m.name != moduleName)
    allModules.push(module)

    // sort modules by name
    allModules.sort((a: Module, b: Module) => {
      if (a.name == b.name) {
        return 0
      }
      return a.name < b.name ? -1 : 1
    })

    this.setState((prevState) => ({
      ...prevState,
      modules: allModules
    }))
  }

  render () {
    const rows = []
    if (this.state.modules.length > 0) {
      this.state.modules.forEach(module => {
        let button = null
        if (module.is_updatable) {
          button = <button type="button"
            className="btn"
            disabled={module.is_loading}
            onClick={() => { this.updateModule(module.name) }}>
            {this.props.language.LC__CMDB__CATG__SMVIEW__UPDATE}
          </button>
        }

        let spinner = null
        if (module.is_loading) {
          spinner = <img src={this.props.spinner_url} alt="..." />
        }

        rows.push(
          <tr key={module.name}>
            <td>{module.title}</td>
            <td>{module.installed_version}</td>
            <td>{module.available_version} {spinner}</td>
            <td>{button}</td>
          </tr>
        )
      })
    }

    return <div id="update_container" className="update-container">
      <table className="mainTable">
        <thead>
          <tr>
            <th>{this.props.language.LC__CMDB__CATG__SMVIEW__ADD_ON_NAME}</th>
            <th>{this.props.language.LC__CMDB__CATG__SMVIEW__CURRENT_VERSION}</th>
            <th>{this.props.language.LC__CMDB__CATG__SMVIEW__AVAILABLE_VERSION}</th>
            <th>&nbsp;</th>
          </tr>
        </thead>
        <tbody>
          {rows}
        </tbody>
      </table>
    </div>
  }
}
