<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */
class isys_cache_smview_memcached extends isys_cache_keyvalue implements isys_cache_keyvaluable
{
    /**
     * @var  Memcached
     */
    protected $m_memcached;

    /**
     * Check wheather memcached is available or not
     *
     * @return  boolean
     */
    public static function available() {
        return class_exists('Memcached');
    }

    /**
     * Delete a cache key.
     *
     * @param   string $p_key
     * @return  isys_cache_smview_memcached
     */
    public function delete($p_key) {
        $this->prepend_ns($p_key);
        $this->m_memcached->delete($p_key);
        return $this;
    }

    /**
     * Determine whether a storage entry has been set for a key.
     *
     * @param   string $key The storage entry identifier.
     * @return  boolean
     */
    public function exists($key) {
        $value = $this->get($key);
        return $value !== false;
    }

    /**
     * Flush cache
     */
    public function flush() {
        $this->m_memcached->flush();
    }

    /**
     *
     * @param   string $p_key
     * @return  mixed
     */
    public function get($p_key) {
        $this->prepend_ns($p_key);

        return $this->m_memcached->get($p_key);
    }

    /**
     * Stores an item var with key on the memcached server. Parameter expire is expiration time in seconds.
     * If it's 0, the item never expires (but memcached server doesn't guarantee this item to be stored all the time,
     * it could be deleted from the cache to make place for other items).
     * You can use MEMCACHE_COMPRESSED constant as flag value if you want to use on-the-fly compression (uses zlib).
     *
     * @param   string  $p_key
     * @param   mixed   $p_value
     * @param   integer $p_ttl [optional] Expiration time of the item. If it's equal to zero, the item will never expire. You can also use Unix timestamp or a number of seconds starting from current time, but in the latter case the number of seconds may not exceed 2592000 (30 days).
     * @return  isys_cache_smview_memcached
     */
    public function set($p_key, $p_value = null, $p_ttl = -1, $use_ns = true) {
        if ($use_ns) {
            $this->prepend_ns($p_key);
        }
        
        $this->m_memcached->set($p_key, $p_value, $this->default_expiration($p_ttl));
        return $this;
    }
    
    public function getMulti($items) {
        // does not support namespaces
        return $this->m_memcached->getMulti($items);
    }

    /**
     * Adds another memcached server.
     *
     * @param  string  $p_host
     * @param  integer $p_port
     * @param  integer $p_weight
     */
    public function add_server($p_host, $p_port, $p_weight = null) {
        $this->m_memcached->addServer($p_host, $p_port, $p_weight);
    }

    /**
     * Construct the memcached and connect to memcache database.
     *
     * @throws isys_exception_cache
     */
    public function __construct() {
        if (class_exists('Memcached')) {
            $l_host = isys_tenantsettings::get('memcache.host', '127.0.0.1');
            $l_port = isys_tenantsettings::get('memcache.port', '11211');

            // add a persistent connection
            $this->m_memcached = new Memcached('shd_smview');
            $this->m_memcached->setOption(Memcached::OPT_LIBKETAMA_COMPATIBLE, true);
            
            // when using persistent connections, it is important to not re-add servers
            if (count($this->m_memcached->getServerList())) {
                return;
            }
            
            if (!$this->m_memcached->addServer($l_host, $l_port)) {
                throw new isys_exception_cache('Could not connect to memcached server on localhost:11211', 'memcached');
            }
        }
        else {
            throw new isys_exception_cache('Memcached is not available. Install the php memcached extension!', 'memcached');
        }
    }
}
