<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */

/**
 * i-doit
 *
 * Template Module Dao.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.org>
 * @version     0.9
 * @copyright   synetics GmbH
 * @license     http://www.i-doit.com/license
 */

define("C__SHD_COLOR__GREY", "#E6E6E6");
define("C__SHD_COLOR__GREEN", "#78D168");
define("C__SHD_COLOR__RED", "#E7A1A2");
define("C__SHD_COLOR__YELLOW", "#FCFA90");
define("C__SHD_COLOR__ORANGE", "#FAB47A");

class smview_cmdb_dao_status extends isys_module_dao
{
    const CACHE_TIMEOUT = 60;
    
    protected static $m_instance = null;

    private $m_monitored_hosts = null;
    private $m_livestatus_hosts = null;
    private $m_livestatus_services = null;

    /**
     * Logger.
     *
     * @var isys_log
     */
    protected $logger = null;
    
    /**
    * The caching instance for monitoring rows.
    *
    * @var  isys_cache_memcached
    */
    protected $m_cache = null;

    public function __construct(isys_component_database $p_db)
    {
        global $g_absdir;
        $this->logger = isys_log::get_instance('shd-smview')
            ->set_log_file($g_absdir . '/log/shd-smview_' . date('Y-m-d') . '.log');

        try {
            $this->m_cache = new isys_cache_smview_memcached();
            $this->m_cache->ns('smview');
        } catch (isys_exception_cache $ex) {
        }

        parent::__construct($p_db);
    }

    /**
     * Retrieve the Map.
     *
     * @param   integer $obj_id
     * @param   boolean $p_with_profile
     *
     * @return  array
     * @throws  Exception
     */
    public function getStatus($obj_id, $p_with_profile = false)
    {
        //Calc Status
        // 2 = Critical !!!     #E7A1A2         215;47;19
        // 3 = Unknown/unreachable  !!      #FAB47A         255;187;85
        // 1 = Warning !        #FCFA90
        // 0 = Okay             #78D168         136;108;102

        // -1 = Nothing -> Grey Status
        // 4 = down == 2 !!!
        // 5 = unreachable == 3 !!!

        // try to get status via livestatus
        try {
            $livestatus = $this->get_livestatus($obj_id);
        } catch (Exception $ex) {
            return [];
        }

        if (! empty($livestatus)) {
            return $livestatus;
        }

        // try to get status via BSM configuration
        $bsmServiceStatus = $this->getBsmServiceStatus($obj_id);
        return $bsmServiceStatus ?: $this->getGreyStatus();
    }

    /**
     * Get BSM service status.
     *
     * @param  int $obj_id
     * @return array|null
     * @throws isys_exception_cache
     */
    public function getBsmServiceStatus($obj_id)
    {
        if (class_exists("bsm_api_model_config")) {
            $param = ["service_id" => $obj_id];
            $bsmConfig = new bsm_api_model_config();
            $bsmServiceStatus = $bsmConfig->status($param);
            if ($bsmServiceStatus["status"] != "-1") {
                if ($bsmServiceStatus["status"] == 0) {
                    $bsmServiceStatus["color"] = C__SHD_COLOR__GREEN;
                } elseif ($bsmServiceStatus["status"] == 1) {
                    $bsmServiceStatus["color"] = C__SHD_COLOR__YELLOW;
                } elseif ($bsmServiceStatus["status"] == 2) {
                    $bsmServiceStatus["color"] = C__SHD_COLOR__RED;
                }
            }
            return $bsmServiceStatus;
        }
        return null;
    }

    public function getGreyStatus()
    {
        return array(
            "text"           => "n/a",
            "status"         => "-1",
            "output"         => "",
            "color"          => C__SHD_COLOR__GREY,
            "service_status" => "0|0|0|0",
        );
    }

    /**
     * This method will retrieve the "livestatus" data of a given host in realtime.
     *
     * @param  int $p_obj_id
     * @return array|null
     * @throws isys_exception_api
     */
    public function get_livestatus($p_obj_id)
    {
        if (! class_exists("livestatus_api_model_host")) {
            return [];
        }

        $identifier = __METHOD__ . "_$p_obj_id";
        if ($this->m_cache) {
            $result = $this->m_cache->get($identifier);
            if ($result) {
                return $result;
            }
        }
        
        $monitored_hosts = $this->getMonitoredHosts();
        $l_monitoring_row = $monitored_hosts[$p_obj_id];
        if (empty($l_monitoring_row)) {
            // this is not a monitored host
            return [];
        }
        $livestatus_api_model_host = new livestatus_api_model_host();
        $hostname = $livestatus_api_model_host->get_hostname_by_monitoringobject($l_monitoring_row);
        // get cached host
        $livestatus = $this->_getLivestatusHost($hostname, $l_monitoring_row);
        if (! isset($livestatus["state"])) {
            $l_return["text"] = "UNKOWN";
            $l_return["color"] = C__SHD_COLOR__ORANGE;
            $l_return["status"] = 3;
            return $l_return;
        }
        
        $l_return["output"] = $livestatus["plugin_output"];
        if ($livestatus["state"] == "0") {
            $l_return["text"] = "OK";
            $l_return["color"] = C__SHD_COLOR__GREEN;
            $l_return["status"] = $livestatus["state"];
        } elseif ($livestatus["state"] == "1") {
            $l_return["text"] = "DOWN";
            $l_return["color"] = C__SHD_COLOR__RED;
            $l_return["status"] = 2;
        } elseif ($livestatus["state"] == "3") {
            $l_return["text"] = "UNKOWN";
            $l_return["color"] = C__SHD_COLOR__ORANGE;
            $l_return["status"] = $livestatus["state"];
        } else {
            $l_return["text"] = "DOWN";
            $l_return["color"] = C__SHD_COLOR__RED;
            $l_return["status"] = 2;
        }

        $l_return["service_status"] = $this->getServiceStatus($l_monitoring_row, $hostname);
        if ($this->m_cache) {
            $this->m_cache->set($identifier, $l_return, self::CACHE_TIMEOUT);
        }
        return $l_return;
    }

    /**
     * This method will retrieve the "livestatus" data of a given host in realtime.
     *
     * @param  int $p_obj_id
     * @return array|null
     */
    public function get_livestatus_info($p_obj_id)
    {
        $identifier = __METHOD__ . "_$p_obj_id";
        if ($this->m_cache) {
            $result = $this->m_cache->get($identifier);
            if ($result) {
                return $result;
            }
        }

        $monitored_hosts = $this->getMonitoredHosts();
        $l_monitoring_row = $monitored_hosts[$p_obj_id];
        return ! empty($l_monitoring_row) ? $l_monitoring_row : [];
    }

    /**
     * Get live status hostname by its hostname selection.
     *
     * @param  int $p_obj_id
     * @return array|null
     */
    public function get_livestatus_hostname($p_obj_id)
    {
        $liveStatusInfo = $this->get_livestatus_info($p_obj_id);
        switch ($liveStatusInfo["isys_catg_monitoring_list__host_name_selection"]) {
            case 0:
                $liveStatusHostname = $liveStatusInfo["isys_catg_monitoring_list__host_name"];
                break;
            case 1:
                $l_api_model_category = new isys_cmdb_dao_category_g_ip(isys_application::instance()->database);
                $data = $l_api_model_category->get_data_as_array(null, $liveStatusInfo["isys_obj__id"], null, null, null);
                $liveStatusHostname = $data[0]["isys_catg_ip_list__hostname"] . "." . $data[0]["isys_catg_ip_list__domain"];
                break;
            case 2:
                $l_api_model_category = new isys_cmdb_dao_category_g_ip(isys_application::instance()->database);
                $data = $l_api_model_category->get_data_as_array(null, $liveStatusInfo["isys_obj__id"], null, null, null);
                $liveStatusHostname = $data[0]["isys_catg_ip_list__hostname"];
                break;
            case 3:
                $liveStatusHostname = $liveStatusInfo["isys_obj__title"];
                break;
            default:
                $liveStatusHostname = "";
        }
        return $liveStatusHostname;
    }


    /**
     * This method will retrieve the "livestatus" data of a given host in realtime.
     *
     * @param  int $p_obj_id
     * @return array|null
     * @throws isys_exception_api
     */
    public function get_services($p_obj_id, $p_services)
    {
        if (! class_exists("livestatus_api_model_service")) {
            return [];
        }

        $livestatusService = new livestatus_api_model_service();
        $livestatusService->set_livestatus_instance_by_host_id($p_obj_id);
        $services = $livestatusService->read(["obj_id" => $p_obj_id]);

        $return_services = [];
        foreach ($services as $service) {
            foreach ($p_services as $searched_services) {
                if ($service["display_name"] == $searched_services) {
                    $return_services[] = $service;
                }
            }
        }

        $l_return = [];
        foreach ($return_services as $service) {
            //hoechster Status gewinnt
            if ($l_return === []) {
                $l_return["status"] = $service["state"];
                $l_return["output"] = $service["plugin_output"];
            } elseif ($service["state"] >= $l_return["status"]) {
                $l_return["status"] = $service["state"];
                $l_return["output"] = $service["plugin_output"];
            }
        }

        if ($l_return["status"] == "0") {
            $l_return["text"] = $l_return["output"];
            $l_return["color"] = C__SHD_COLOR__GREEN;
        } elseif ($l_return["state"] == "2") {
            $l_return["text"] = $l_return["output"];
            $l_return["color"] = C__SHD_COLOR__RED;
        } elseif ($l_return["state"] == "3") {
            $l_return["text"] = "UNKOWN";
            $l_return["color"] = C__SHD_COLOR__ORANGE;
        } else {
            $l_return["text"] = "DOWN";
            $l_return["color"] = C__SHD_COLOR__RED;
        }
        $l_return["services"] = $return_services;
        return $l_return;
    }

    /**
     * This method will retrieve the "livestatus" data of a given host in realtime.
     *
     * @param  int $p_obj_id
     * @return array|null
     */
    public function get_all_services_by_hostname($p_obj_id)
    {
        if (! class_exists("livestatus_api_model_service")) {
            return [];
        }

        try {
            $livestatusService = new livestatus_api_model_service();
            $livestatusService->set_livestatus_instance_by_host_id($p_obj_id);
            $hostname = $this->get_livestatus_hostname($p_obj_id);
            return !empty($hostname) ? $livestatusService->read(["hostname" => $hostname]) : [];
        }
        catch ( Exception $ex) {
            return [];
        }
    }

    /**
     * This method will retrieve the "livestatus" data of a given host in realtime.
     *
     * @param  int $p_obj_id
     * @return array|null
     * @throws isys_exception_api
     */
    public function get_services_by_hostname($p_obj_id, $p_services)
    {
        $services = $this->get_all_services_by_hostname($p_obj_id);
        $return_services = [];
        foreach ($services as $service) {
            foreach ($p_services as $searched_services) {
                if ($service["display_name"] == $searched_services) {
                    $return_services[] = $service;
                }
            }
        }

        $l_return = [];
        foreach ($return_services as $service) {
            //hoechster Status gewinnt
            if ($l_return === []) {
                $l_return["status"] = $service["state"];
                $l_return["output"] = $service["plugin_output"];
                continue;
            }
            //always set Critical
            if ($service["state"] == "2") {
                $l_return["status"] = $service["state"];
                $l_return["output"] = $service["plugin_output"];
                continue;
            }

            if ($service["state"] >= $l_return["status"] && $l_return["status"] != 2) {
                $l_return["status"] = $service["state"];
                $l_return["output"] = $service["plugin_output"];
                continue;
            }

            if ($service["state"] == 1 && !$l_return["status"] >= 2) {
                $l_return["status"] = $service["state"];
                $l_return["output"] = $service["plugin_output"];
            }
        }

        if ($l_return["status"] == "0") {
            $l_return["text"] = $l_return["output"];
            $l_return["color"] = C__SHD_COLOR__GREEN;
        } elseif ($l_return["status"] == "1") {
            $l_return["text"] = $l_return["output"];
            $l_return["color"] = C__SHD_COLOR__YELLOW;
        } elseif ($l_return["status"] == "2") {
            $l_return["text"] = $l_return["output"];
            $l_return["color"] = C__SHD_COLOR__RED;
        } elseif ($l_return["status"] == "3") {
            $l_return["text"] = "UNKOWN";
            $l_return["color"] = C__SHD_COLOR__ORANGE;
        } else {
            $l_return["text"] = "DOWN";
            $l_return["color"] = C__SHD_COLOR__RED;
        }

        $l_return["services"] = $return_services;
        return $l_return;
    }

    private function getServiceStatus($l_monitoring_row, $hostname)
    {
        $hosts = $this->getAllLivestatusServices($l_monitoring_row);
        $list = isset($hosts[$hostname]) ? $hosts[$hostname] : [];
        
        // create selection of all livestatus services
        $l_services = ["0" => "0", "1" => "0", "2" => "0", "3" => "0"];
        foreach ($list as $service) {
            $l_services[$service['state']] = $l_services[$service['state']] + 1;
        }
        return $l_services[2] . "|" .$l_services[3]. "|" .$l_services[1]. "|" .$l_services[0];
    }

    /**
     * Get monitored hosts.
     *
     * @return array
     */
    public function getMonitoredHosts()
    {
        if ($this->m_monitored_hosts != null) {
            return $this->m_monitored_hosts;
        }

        // get monitoring instance of this host
        $l_monitoring_res = isys_cmdb_dao_category_g_monitoring::instance(isys_application::instance()->database)->get_data();

        $monitored_hosts = [];
        while ($l_monitoring_row = $l_monitoring_res->get_row()) {
            if ($l_monitoring_row['isys_catg_monitoring_list__active'] == '1') {
                $monitored_hosts[$l_monitoring_row["isys_obj__id"]] = $l_monitoring_row;
            }
        }
        
        $this->m_monitored_hosts = $monitored_hosts;
        return $monitored_hosts;
    }

    /**
     * Get a livestatus hosts.
     *
     * @return array|null
     * @throws isys_exception_api
     */
    private function _getLivestatusHost($hostname, $l_monitoring_row)
    {
        if (! class_exists("livestatus_api_model_host")) {
            return null;
        }

        $instance_id = $l_monitoring_row['isys_monitoring_hosts__id'];
        
        $this->logger->debug(__METHOD__ . ": get livestatus host: $hostname");
        if ($this->m_livestatus_hosts == null || empty($this->m_livestatus_hosts[$instance_id])) {
            $this->logger->debug(__METHOD__ . ": get all hosts from livestatus");
            
            $livestatusHost = new livestatus_api_model_host();
            $livestatusHost->set_livestatus_instance($l_monitoring_row);
            $hosts = $livestatusHost->read();
            $this->m_livestatus_hosts[$instance_id] = $hosts;
        }
        
        $hosts = $this->m_livestatus_hosts[$instance_id];
        foreach ($hosts as $host) {
            if ($host['name'] == $hostname) {
                return $host;
            }
        }
        return null;
    }

    /**
     * Get all livestatus services.
     *
     * @param  array $l_monitoring_row
     * @return array
     * @throws isys_exception_api
     */
    private function getAllLivestatusServices($l_monitoring_row)
    {
        if (! class_exists("livestatus_api_model_service")) {
            return [];
        }

        $instance_id = $l_monitoring_row['isys_monitoring_hosts__id'];
        if ($this->m_livestatus_services != null && isset($this->m_livestatus_services[$instance_id])) {
            return $this->m_livestatus_services[$instance_id];
        }
        
        $livestatusService = new livestatus_api_model_service();
        $livestatusService->set_livestatus_instance($l_monitoring_row);
        $services = $livestatusService->get_all_services();
        $this->m_livestatus_services[$instance_id] = $services;
        return $services;
    }

    /**
     * get_data always retrieves the data of the main table of this module
     */
    public function get_data()
    {
        // TODO: Implement get_data() method.
    }
}
