<?php

use idoit\Module\License\Exception\LegacyLicenseExistingException;
use idoit\Module\License\Exception\LegacyLicenseExpiredException;
use idoit\Module\License\Exception\LegacyLicenseInstallException;
use idoit\Module\License\Exception\LegacyLicenseInvalidKeyException;
use idoit\Module\License\Exception\LegacyLicenseInvalidTypeException;
use idoit\Module\License\Exception\LegacyLicenseParseException;
use idoit\Module\License\LicenseServiceFactory;

/**
 * SM-VIEW license installation controller.
 *
 * @package    idoit
 * @subpackage Livestatus
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_handler_smview_license_installation extends isys_handler
{
    /**
     * Path to license file.
     * 
     * @var string
     */
    private $license_file_path = '/idoit-license/i-doit.key';
    
    /**
     * ID of mandator.
     * 
     * @var int
     */
    private $l_mandator_id = -1;
    
    /**
     * License module.
     * 
     * @var isys_module_licence
     */
    private $l_licence;
    
    /**
     * Application instance.
     * 
     * @var isys_application
     */
    private $app;
    
    /**
     * Constructor.
     */
    public function __construct() 
    {
        $this->l_licence = new isys_module_licence();
        $this->app = isys_application::instance();
    }
    
    /**
     * Initialization method.
     * 
     * You can call this controller via CLI from i-doit root:
     * sudo -u www-data ./controller -v -d -u admin -p admin -i 1 -m smview_license_installation
     *
     */
    public function init() 
    {
        global $g_absdir, $l_dao_mandator, $g_mandator;
        
        verbose("start\n");
        
        // get mandator, set by console parameter
        $this->l_mandator_id = ! empty($g_mandator) ? $g_mandator : 1;
        verbose("using mandator: " . $this->l_mandator_id . "\n");
        
        if (!is_file($this->license_file_path)) {
            verbose("could not find license file\n");
            return;
        }
        
        // include admin functions and get instance of mandator database
        include_once $g_absdir . "/admin/src/functions.inc.php";
        $p_database = $this->app->database_system;
        $l_dao_mandator = new isys_component_dao_mandator($p_database);
        connect_mandator($this->l_mandator_id);
        
        // install license
        $this->_install_license();
        
        verbose("finished\n");
    }
    
    /**
     * Install license.
     */
    private function _install_license()
    {
        /*
         * Code copied and modified from:
         * 
         * class: isys_module_system.class.php
         * method: handle_licence_installation
         */

        global $g_license_token;
        
        // multi-tenant
        if (! class_exists('isys_module_licence')) {
            verbose("isys_module_licence does not exist\n");
            return;
        }

        $licenseService = LicenseServiceFactory::createDefaultLicenseService(isys_application::instance()->container->get('database_system'), $g_license_token);

        // We try to catch a certain exception.
        try {
            $l_lic_parse = $licenseService->parseLicenseFile($this->license_file_path);
        } catch (LegacyLicenseExpiredException $e) {
            verbose("invalid license file: " . $e->getMessage() . "\n");
            return;
        } catch (LegacyLicenseInvalidKeyException $e) {
            verbose("invalid license file: " . $e->getMessage() . "\n");
            return;
        } catch (LegacyLicenseInvalidTypeException $e) {
            verbose("invalid license file: " . $e->getMessage() . "\n");
            return;
        } catch (LegacyLicenseParseException $e) {
            verbose("invalid license file: " . $e->getMessage() . "\n");
            return;
        }
        
        if (! is_array($l_lic_parse)) {
            verbose("invalid license data\n");
            return;
        }

        try {
            $licenseService->installLegacyLicense($l_lic_parse);

            verbose("installation successful\n");
        } catch (LegacyLicenseInstallException $e) {
            verbose("installation failed:");
            verbose($e->getMessage() . "\n");
        } catch (LegacyLicenseExistingException $e) {
            verbose("installation failed, because license already exists:");
            verbose($e->getMessage() . "\n");
        } catch (LegacyLicenseExpiredException $e) {
            verbose("installation failed, because license has expired:");
            verbose($e->getMessage() . "\n");
        }
    }
}
