<?php

namespace idoit\Module\Smview\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use isys_application;
use isys_component_dao_mandator;
use idoit\Module\License\Exception\LegacyLicenseExistingException;
use idoit\Module\License\Exception\LegacyLicenseExpiredException;
use idoit\Module\License\Exception\LegacyLicenseInstallException;
use idoit\Module\License\Exception\LegacyLicenseInvalidKeyException;
use idoit\Module\License\Exception\LegacyLicenseInvalidTypeException;
use idoit\Module\License\Exception\LegacyLicenseParseException;
use idoit\Module\License\LicenseServiceFactory;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Command to install an i-doit license.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class LicenseInstallationCommand extends AbstractCommand
{

    /**
     * Path to license file.
     *
     * @var string
     */
    private $license_file_path = '/idoit-license/i-doit.key';

    /**
     * ID of mandator.
     *
     * @var int
     */
    private $l_mandator_id = -1;

    /**
     * License module.
     *
     * @var \isys_module_licence
     */
    private $l_licence;

    /**
     * Application instance.
     *
     * @var isys_application
     */
    private $app;

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName(): string
    {
        return 'smview_license_installation';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription(): string
    {
        return 'Command to install an i-doit license.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition(): InputDefinition
    {
        return new InputDefinition();
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable(): bool
    {
        return false;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        global $g_absdir, $l_dao_mandator, $g_mandator;
        try {
            $output->writeln("start\n");
            $output->writeln("");

            // get mandator, set by console parameter
            $this->l_mandator_id = ! empty($g_mandator) ? $g_mandator : 1;
            $output->writeln("using mandator: " . $this->l_mandator_id . "\n");

            if (!is_file($this->license_file_path)) {
                $output->writeln("could not find license file\n");
                return 1;
            }

            // include admin functions and get instance of mandator database
            include_once $g_absdir . "/admin/src/functions.inc.php";
            $p_database = $this->app->database_system;
            $l_dao_mandator = new isys_component_dao_mandator($p_database);
            connect_mandator($this->l_mandator_id);

            // install license
            $this->_install_license($output);

            $output->writeln("finished\n");
        } catch (Exception $e) {
            $output->writeln("");
            $output->writeln($e);
            $output->writeln("");
        }
        return 0;
    }

    /**
     * Install license.
     */
    private function _install_license($output)
    {
        /*
         * Code copied and modified from:
         *
         * class: isys_module_system.class.php
         * method: handle_licence_installation
         */

        global $g_license_token;

        // multi-tenant
        if (! class_exists('isys_module_licence')) {
            $output->writeln("isys_module_licence does not exist\n");
            return;
        }

        $licenseService = LicenseServiceFactory::createDefaultLicenseService(isys_application::instance()->container->get('database_system'), $g_license_token);

        // We try to catch a certain exception.
        try {
            $l_lic_parse = $licenseService->parseLicenseFile($this->license_file_path);
        } catch (LegacyLicenseExpiredException | LegacyLicenseInvalidKeyException | LegacyLicenseInvalidTypeException | LegacyLicenseParseException $e) {
            $output->writeln("invalid license file: " . $e->getMessage() . "\n");
            return;
        }

        if (! is_array($l_lic_parse)) {
            $output->writeln("invalid license data\n");
            return;
        }

        try {
            $licenseService->installLegacyLicense($l_lic_parse);

            $output->writeln("installation successful\n");
        } catch (LegacyLicenseInstallException $e) {
            $output->writeln("installation failed:");
            $output->writeln($e->getMessage() . "\n");
        } catch (LegacyLicenseExistingException $e) {
            $output->writeln("installation failed, because license already exists:");
            $output->writeln($e->getMessage() . "\n");
        } catch (LegacyLicenseExpiredException $e) {
            $output->writeln("installation failed, because license has expired:");
            $output->writeln($e->getMessage() . "\n");
        }
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'smview_license_installation -u admin -p admin -i 1',
        ];
    }
}
