<?php

namespace idoit\Module\Smview;

use Exception;
use isys_component_dao_mandator;
use isys_component_database;
use ZipArchive;

/**
 * Module updater class.
 *
 * @package    idoit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class ModuleUpdater {
    
    const SEVERITY_VERBOSE = 'verbose';
    const SEVERITY_ERROR = 'error';
    
    /**
     * Output progress to console.
     * 
     * @var boolean
     */
    private $_console_output = false;
    
    /**
     * Download URL.
     * 
     * @var string
     */
    private $_download_url = 'https://www.sm-view.de/download/';
    
    /**
     * Preview Download URL.
     * 
     * @var string
     */
    private $_preview_download_url = 'https://www.sm-view.de/preview/';
    
    /**
     * Download directory.
     * 
     * @var string
     */
    private $_download_dir = '/var/www/html/i-doit/temp/';
    
    /**
     * Available modules.
     * 
     * @var array
     */
    private $_available_module_classes = array(
        'isys_module_bsm'              => 'bsm',
        'isys_module_bvg'              => 'bvg',
        'isys_module_livestatus'       => 'livestatus',
        'isys_module_smview'           => 'smview',
        'isys_module_smview_cable'     => 'smview_cable',
        'isys_module_smview_feed'      => 'smview_feed',
        'isys_module_smview_maps'      => 'smview_maps',
        'isys_module_smview_patch'     => 'smview_patch',
        'isys_module_smview_reporting' => 'smview_reporting',
        'isys_module_ukd'              => 'ukd',
        'isys_module_ukl'              => 'ukl',
        'isys_module_zit_importer'     => 'zit_importer',
    );
    
    /**
     * Constructor.
     * 
     * @param boolean $console_output
     */
    public function __construct($console_output=false) {
        $this->_console_output = $console_output;
    }
    
    /**
     * Get available modules, which can be updated.
     * 
     * @return array
     */
    public function getAvailableModules() {
        return $this->_available_module_classes;
    }
    
    /**
     * Get current installed module version.
     * 
     * @global string $g_absdir
     * @param string $module
     * @return string|null
     */
    public function getCurrentModuleVersion($module) {
        global $g_absdir;
        
        $dir = $g_absdir . '/src/classes/modules/' . $module . '/';
        if (! is_dir($dir) || ! file_exists($dir . 'package.json')) {
            return null;
        }
        
        $l_package = json_decode(file_get_contents($dir . 'package.json'), true);
        return $l_package['version'];
    }
    
    /**
     * Get available module version by downloading from server.
     * 
     * @param string $module
     * @return string|null
     */
    public function getAvailableModuleVersion($module, $usePreview=false) {
        $downloadDir = $this->_download_dir . 'test/';
        if (!is_dir($downloadDir)) {
            mkdir($downloadDir);
        }
        
        // download module
        $this->_downloadModule($module . '.zip', $usePreview, $downloadDir);
        
        if (!class_exists('ZipArchive') || !extension_loaded('zlib')) {
            return null;
        }
        
        // unzip
        $zipFile = $downloadDir . $module . '.zip';
        $l_zip = new ZipArchive();
        if ($l_zip->open($zipFile) === true) {
            $l_zip->extractTo($downloadDir, false ?: null);
            $l_zip->close();
        }
        
        // get package information
        if (!file_exists($downloadDir . '/package.json')) {
            exec("rm -rf $downloadDir");
            return null;
        }

        $l_package = json_decode(file_get_contents($downloadDir . '/package.json'), true);
        $version = $l_package['version'];
        
        // remove temporary directory
        exec("rm -rf $downloadDir");
        
        return $version;
    }
    
    /**
     * Start update of specified modules.
     * 
     * @global string $g_absdir
     * @global isys_component_database $g_comp_database_system
     * @global isys_component_dao_mandator $l_dao_mandator
     * @param array $modules
     * @param boolean $usePreview
     * @return array
     */
    public function update($modules, $usePreview=false) {
        global $g_absdir, $g_comp_database_system, $l_dao_mandator;
        
        $this->_writeToConsole("Start module updater\n", self::SEVERITY_VERBOSE);
        
        // get installed modules
        $installedModules = [];
        foreach ($this->_available_module_classes as $className => $moduleName) {
            if (! in_array($moduleName, $modules)) {
                # this module should not be updated
                continue;
            }
            
            $installed_version = $this->getCurrentModuleVersion($moduleName);
            $available_version = $this->getAvailableModuleVersion($moduleName, $usePreview);

            $cmp = version_compare($installed_version, $available_version);
            if ($cmp >= 0) {
                # installed version is newer
                continue;
            }
            
            if (class_exists($className)) {
                $this->_writeToConsole("Found installed module $moduleName", self::SEVERITY_VERBOSE);
                $installedModules[$moduleName] = $moduleName . '.zip';
            }
        }
        
        // try to download zip-file for each installed module
        foreach ($installedModules as $module) {
            $this->_downloadModule($module, $usePreview);
        }
        
        $this->_writeToConsole("Start module import\n", self::SEVERITY_VERBOSE);

        // instanciate mandator DAO, needed within functions.inc.php
        $l_dao_mandator = new isys_component_dao_mandator($g_comp_database_system);
        include_once $g_absdir . "/admin/src/functions.inc.php";
        
        // try to install or update each module
        foreach ($installedModules as $module) {
            $this->_installModule($module);
        }
        
        $this->_writeToConsole("Clear constant cache\n", self::SEVERITY_VERBOSE);
        
        // cleanup temp directory (otherwise i-doit will not work properly)
        $temp_path = $g_absdir . '/temp/';
        exec("rm -rf $temp_path");
        
        $this->_writeToConsole("Finished module import\n", self::SEVERITY_VERBOSE);
        return $installedModules;
    }
    
    /**
     * Download module.
     * 
     * @param string $module
     * @param boolean $usePreview
     * @param string $downloadDir
     */
    private function _downloadModule($module, $usePreview=false, $downloadDir=null) {
        $module_path = isset($downloadDir) ? $downloadDir : $this->_download_dir;
        
        $this->_writeToConsole("Downloading module $module ...", self::SEVERITY_VERBOSE);
        $url = $this->_download_url;
        if ($usePreview) {
            $url = $this->_preview_download_url;
        }
        $url .= $module;
        $zip = file_get_contents($url);
        
        if ($zip === false) {
            $this->_writeToConsole("Can not download from: $url \n", self::SEVERITY_ERROR);
            return;
        }

        $file_path = $module_path . $module;
        if (file_exists($file_path)) {
            return;
        }

        if (!is_writable($module_path)) {
            $this->_writeToConsole("Can not write to filesystem: $module_path \n", self::SEVERITY_ERROR);
            return;
        }
        $fp = fopen($file_path, 'w');
        fwrite($fp, $zip);
        fclose($fp);

        if (!file_exists($file_path)) {
            $this->_writeToConsole("Could not write module file: $file_path \n", self::SEVERITY_ERROR);
        }
    }
    
    /**
     * Install module.
     * 
     * @param string $module
     */
    private function _installModule($module) {
        $module_path = $this->_download_dir;
        $zipFile = $module_path . $module;
        $this->_writeToConsole("Installing module '$zipFile'\n", self::SEVERITY_VERBOSE);

        try {
            $result = install_module_by_zip($zipFile);
        } catch (Exception $ex) {
            $message = $ex->getMessage();
            $this->_writeToConsole("Error: while installing module: $message\n", self::SEVERITY_ERROR);
            return;
        }

        if ($result) {
            $this->_writeToConsole("Module successfully installed\n", self::SEVERITY_VERBOSE);
        }
    }
    
    /**
     * Write output to console if enabled.
     * 
     * @param string $text
     * @param string $severity
     */
    private function _writeToConsole($text, $severity) {
        if (! $this->_console_output) {
            return;
        }
        
        switch ($severity) {
            case self::SEVERITY_ERROR:
                error($text);
                break;
            case self::SEVERITY_VERBOSE:
                verbose($text);
                break;
        }
    }
}
