<?php

/**
 * i-doit
 *
 * API test
 *
 * @package    i-doit
 * @subpackage unit_testing
 * @copyright  synetics GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_json_rpc
{

    /**
     * Apache response headers
     *
     * @var array
     */
    private static $m_headers = null;

    /**
     * The request id
     *
     * @var integer
     */
    private $id;

    /**
     * Structure array(
     *   'username' => 'xyz',
     *   'password' => 'xyz',
     *   'sessionID' => '123'
     * )
     *
     * @var array
     */
    private $m_auth = [];

    /**
     * Debug state
     *
     * @var boolean
     */
    private $m_debug;

    /**
     * @var string
     */
    private $m_rpcVersion = '2.0';

    /**
     * The server URL
     *
     * @var string
     */
    private $m_url;

    /**
     * If true, notifications are performed instead of requests
     *
     * @var boolean
     */
    private $notification = false;

    /**
     * Retrieve HTTP headers.
     *
     * @return  array
     */
    public static function headers($p_headers)
    {
        $nHeaders = [];
        foreach ($p_headers as $header) {
            $h = explode(": ", $header);
            if (isset($h[1])) {
                $nHeaders[$h[0]] = $h[1];
            }
        }

        return $nHeaders;
    }

    /**
     * @param $p_onoff
     *
     * @return $this
     */
    public function setDebug($p_onoff)
    {
        $this->m_debug = $p_onoff;

        return $this;
    }

    /**
     * Sets the notification state of the object. In this state, notifications are performed, instead of requests.
     *
     * @param boolean $notification
     */
    public function setRPCNotification($notification)
    {
        empty($notification) ? $this->notification = false : $this->notification = true;
    }

    /**
     * Login via RPC or Basic Auth
     *
     *  For Basic Auth: use $p_authtype = 'basic'
     *  For RPC Auth:   use $p_authtype = 'rpc'
     *
     * @param strign $p_username
     * @param strign $p_password
     * @param string $p_authType
     *
     * @return JSONRPC
     */
    public function setRPCAuth($p_username, $p_password, $p_authType = 'rpc')
    {
        $this->m_auth['username'] = $p_username;
        $this->m_auth['password'] = $p_password;
        $this->m_auth['type'] = $p_authType;

        return $this;
    }

    /**
     * @param string $p_sessionID
     *
     * @return JSONRPC
     */
    public function setSessionID($p_sessionID)
    {
        $this->m_auth['sessionID'] = $p_sessionID;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getSessionID()
    {
        return isset($this->m_auth['sessionID']) ? $this->m_auth['sessionID'] : null;
    }

    /**
     * Performs a jsonRCP request and gets the results as an array
     *
     * @param string $method
     * @param array  $params
     *
     * @return array
     */
    public function __call($method, $params = [])
    {

        // check
        if (!is_scalar($method)) {
            throw new Exception('Method name has no scalar value');
        }

        // check
        if (!is_array($params)) {
            throw new Exception('Params must be given as array');
        }

        // sets notification or request task
        if ($this->notification) {
            $currentId = null;
        } else {
            $currentId = $this->id;
        }

        // prepares the request
        $request = [
            'method'  => $method,
            'params'  => $params,
            'id'      => $currentId,
            'version' => $this->m_rpcVersion
        ];

        $request = json_encode($request);

        // performs the HTTP POST
        $opts = [
            'http' => [
                'method'  => 'POST',
                'header'  => $this->buildHeaders(),
                'content' => $request
            ],
            'ssl' => [
                'verify_peer'=>false,
                'verify_peer_name'=>false,
            ]
        ];

        $l_debug = '';

        if ($this->m_debug) {
            $l_debug .= '***** Request *****' . "\n" . $request . "\n" . '***** End Of request *****' . "\n\n";
        }

        $context = stream_context_create($opts);
        if ($fp = fopen($this->m_url, 'r', false, $context)) {
            $h = $this->headers($http_response_header);
            if (isset($h['X-RPC-Auth-Session'])) {
                $this->m_auth['sessionID'] = $h['X-RPC-Auth-Session'];
            }

            $response = '';
            while ($row = fgets($fp)) {
                $response .= trim($row) . "\n";
            }
            if ($this->m_debug) {
                $l_debug .= '***** Server response *****' . "\n" . $response . '***** End of server response *****' . "\n";
            }
            $response = json_decode($response, true);
        } else {
            throw new Exception('Unable to connect to ' . $this->m_url);
        }

        // debug output
        if ($this->m_debug) {
            echo nl2br($l_debug);
        }

        // final checks and return
        if (!$this->notification) {
            // check
            if ($response['id'] != $currentId) {
                throw new Exception('Incorrect response id (request id: ' . $currentId . ', response id: ' . $response['id'] . ')');
            }
            if (isset($response['error']) && !is_null($response['error'])) {
                throw new Exception('Request error: ' . $response['error']['message']);
            }

            return $response['result'];

        } else {
            return true;
        }
    }

    /**
     * @return string
     */
    private function buildHeaders()
    {
        $headers = [
            "Content-type" => "application/json"
        ];
        $rawHeader = "";

        if (!empty($this->m_auth)) {
            if (isset($this->m_auth['sessionID'])) {
                $headers["Cookie"] = 'PHPSESSID=' . $this->m_auth['sessionID'];
            } else {
                switch ($this->m_auth['type']) {
                    case 'basic':
                        $headers['Authorization'] = sprintf('Basic %s', base64_encode($this->m_auth['username'] . ':' . $this->m_auth['password']));
                        break;
                    default:
                        $headers["X-RPC-Auth-Username"] = $this->m_auth['username'];
                        $headers["X-RPC-Auth-Password"] = $this->m_auth['password'];
                        break;
                }
            }
        }
        foreach ($headers as $h => $c) {
            $rawHeader .= $h . ": " . $c . "\r\n";
        }

        return $rawHeader;
    }

    /**
     * Takes the connection parameters
     *
     * @param string  $url
     * @param boolean $debug
     */
    public function __construct($url, $debug = false)
    {
        // server URL
        $this->m_url = $url;
        // proxy
        empty($proxy) ? $this->proxy = '' : $this->proxy = $proxy;
        // debug state
        empty($debug) ? $this->m_debug = false : $this->m_debug = true;
        // message id
        $this->id = 1;
    }
}